import 'dart:async';

import 'package:fis_i18n/i18n.dart';
import 'package:fis_jsonrpc/rpc.dart';
import 'package:fis_lib_qrcode/qr_flutter.dart';
import 'package:fis_measure/interfaces/process/player/play_controller.dart';
import 'package:fis_measure/interfaces/process/standard_line/calibration.dart';
import 'package:fis_measure/interfaces/process/workspace/application.dart';
import 'package:fis_measure/interfaces/process/workspace/measure_3d_view_controller.dart';
import 'package:fis_measure/process/workspace/measure_3d_view_controller.dart';
import 'package:fis_measure/process/workspace/measure_data_controller.dart';
import 'package:fis_measure/process/workspace/measure_handler.dart';
import 'package:fis_measure/utils/prompt_box.dart';
import 'package:fis_measure/view/paint/ai_patint_controller.dart';
import 'package:fis_measure/view/player/controller.dart';
import 'package:fis_ui/index.dart';

import 'package:flutter/material.dart';
import 'package:get/get.dart';
import 'package:url_launcher/url_launcher.dart';

class ButtonGroup extends StatefulWidget {
  final VoidCallback capturePng;
  const ButtonGroup({
    Key? key,
    required this.capturePng,
  }) : super(key: key);

  @override
  State<ButtonGroup> createState() => _ButtonGroupState();
}

class _ButtonGroupState extends State<ButtonGroup> {
  double _width = 0;
  bool _isExpanded = false;
  final _key = GlobalKey();
  final application = Get.find<IApplication>();

  /// 数据
  final measureData = Get.find<MeasureDataController>();
  final playerController = Get.find<IPlayerController>() as VidPlayerController;
  final measure3DViewController = Get.find<Measure3DViewController>();
  late final measureHandler = Get.find<MeasureHandler>();
  late final aiPatintController = Get.find<AiPatintController>();
  late bool canShowAI = [
    DiagnosisConclusionEnum.Benign,
    DiagnosisConclusionEnum.Malignant,
    DiagnosisConclusionEnum.BenignAndMalignant
  ].contains(measureData.diagnosisConclusion);

  bool ifShowModeButton = false;
  bool ifShowVidModeButton = true;
  get isShell => measure3DViewController.isShell;
  get exist3DData => measure3DViewController.exist3DData;

  @override
  void initState() {
    super.initState();
    measure3DViewController.updatePlayerMode.addListener(_onModeChanged);
  }

  @override
  void dispose() {
    measure3DViewController.updatePlayerMode.removeListener(_onModeChanged);
    super.dispose();
  }

  /// 模式改变触发更新
  void _onModeChanged(Object s, MeasureMode mode) {
    switch (mode) {
      case MeasureMode.vidMode:
        setState(() {
          ifShowModeButton = false;
          ifShowVidModeButton = true;
        });
        break;
      case MeasureMode.carotid2DMode:
        setState(() {
          ifShowModeButton = true;
          ifShowVidModeButton = false;
        });
        break;
      case MeasureMode.carotid3DMode:
        setState(() {
          ifShowModeButton = false;
        });
        break;
    }
  }

  List<Widget> buildTitleButtonList() {
    return [
      _buildTitleButton(
        FIcons.fis_quash,
        i18nBook.common.revoke.t,
        () => application.undoRecord(),
      ),
      _buildTitleButton(
        FIcons.fis_purge,
        i18nBook.measure.clear.t,
        () => application.clearRecords(),
      ),
      _buildTitleButton(
        measureHandler.fullScreenState
            ? FIcons.fis_full_screen_reduction
            : FIcons.fis_full_screen,
        measureHandler.fullScreenState
            ? i18nBook.common.cancel.t + i18nBook.measure.fullScreen.t
            : i18nBook.measure.fullScreen.t,
        () {
          measureHandler.fullScreenState = !measureHandler.fullScreenState;
          setState(() {});
        },
      ),
      _buildTitleButton(
        FIcons.fis_save,
        i18nBook.common.save.t,
        widget.capturePng,
      ),
      _buildTitleButton(
        FIcons.fis_share,
        i18nBook.remedical.share.t,
        () async {
          var itemCurrentImage = await measureData.shareImage.call(
            measureData.itemCurrentImage,
          );
          Get.dialog(_buildShareQr(itemCurrentImage));
        },
      ),
      if (application.isThirdPart) ...[
        _buildTitleButton(
          FIcons.device,
          i18nBook.remedical.calibrationLine.t,
          () {
            Get.find<IStandardLineCalibrationController>().enterEditMode();
          },
        ),
      ],
      if (canShowAI) ...[
        _buildTitleButton(
          !aiPatintController.state.ifShowAi
              ? FIcons.fis_eye_display
              : FIcons.fis_eye_hidden,
          !aiPatintController.state.ifShowAi
              ? i18nBook.measure.showAi.t
              : i18nBook.measure.hideAi.t,
          () {
            aiPatintController.state.ifShowAi =
                !aiPatintController.state.ifShowAi;
            setState(() {});
          },
        ),
      ],
      if (isShell && exist3DData && ifShowVidModeButton) ...[
        _buildTitleButton(
          FIcons.three_dimensional,
          i18nBook.measure.carotid3DMode.t,
          () {
            measure3DViewController.changeModeTo3DMode();
          },
        ),
      ],
      if (isShell && exist3DData && ifShowModeButton) ...[
        _buildTitleButton(
          FIcons.three_dimensional,
          i18nBook.measure.carotid3DMode.t,
          () {
            measure3DViewController.backTo3DMode();
          },
        ),
        _buildTitleButton(
          Icons.play_circle_outline_rounded,
          i18nBook.measure.vidMode.t,
          () {
            measure3DViewController.backToVidMode();
            playerController.resetCurrentFrame();
          },
        ),
      ]
    ];
  }

  @override
  Widget build(BuildContext context) {
    return Row(
      mainAxisAlignment: MainAxisAlignment.end,
      children: [
        InkWell(
          onTap: () {},
          onHover: (isHover) {
            if (!isHover) {
              setState(() {
                _width = 0;
                _isExpanded = isHover;
              });
            } else {
              setState(() {
                // application.isThirdPart ? _width = 270 : _width = 220;
                _width = buildTitleButtonList().length * 48;
                _isExpanded = isHover;
              });
            }
          },
          child: AnimatedContainer(
            duration: const Duration(milliseconds: 300),
            padding: const EdgeInsets.only(
              top: 5,
              bottom: 5,
              left: 15,
            ),
            decoration: BoxDecoration(
              borderRadius: const BorderRadius.only(
                topLeft: Radius.circular(4),
                bottomLeft: Radius.circular(4),
              ),
              color: Colors.grey.withOpacity(0.6),
            ),
            width: _width,
            height: 60,
            child: _isExpanded
                ? FutureBuilder(
                    future: _showList(),
                    builder: (context, snapshot) {
                      if (!snapshot.hasData) {
                        return const SizedBox();
                      }
                      return Row(
                        children: buildTitleButtonList(),
                      );
                    },
                  )
                : const SizedBox(),
          ),
        ),
        InkWell(
          onTap: () {},
          onHover: (isHover) {
            if (isHover) {
              setState(() {
                _width = buildTitleButtonList().length * 48;
                _isExpanded = isHover;
              });
            } else {
              setState(() {
                _width = 0;
                _isExpanded = isHover;
              });
            }
          },
          child: Container(
            height: 60,
            color: Colors.white,
            child: !_isExpanded
                ? const Icon(Icons.chevron_left_rounded)
                : const Icon(Icons.chevron_right_rounded),
          ),
        ),
      ],
    );
  }

  FWidget _buildShareQr(String itemCurrentImage) {
    return FSimpleDialog(
      title: FText(
        i18nBook.remedical.share.t,
        style: const TextStyle(
          color: Colors.white,
          fontSize: 18,
        ),
      ),
      isDefault: false,
      children: [
        FContainer(
          width: 360,
          child: FContainer(
            padding: const EdgeInsets.only(top: 20, bottom: 20),
            child: QRCodeWithLogo(
              itemCurrentImage,
              codeStatement: i18nBook.remedical.scanQrCodeToShareImage.t,
              operationStatement: i18nBook.remedical.copyLink.t,
              operationSuccessCallback: () =>
                  {PromptBox.toast(i18nBook.remedical.copySuccess.t)},
            ),
          ),
        ),
      ],
    );
  }

//定义方法
  void _launchURL(url) async {
    if (await canLaunch(url)) {
      await launch(url);
    } else {
      throw 'Could not launch $url';
    }
  }

//跳转
  Future<bool> _showList() async {
    await Future.delayed(const Duration(milliseconds: 300));
    return true;
  }

  Widget _buildTitleButton(IconData icon, String title, Function onClick) {
    return Container(
      margin: const EdgeInsets.only(
        right: 10,
      ),
      child: InkWell(
        onTap: () => onClick.call(),
        child: Column(
          children: [
            Icon(
              icon,
              color: Colors.white,
            ),
            Text(
              title,
              style: const TextStyle(
                color: Colors.white,
                fontSize: 14,
              ),
            )
          ],
        ),
      ),
    );
  }
}
